"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.create = void 0;
const language_core_1 = require("@vue/language-core");
const shared_1 = require("@vue/shared");
const path = require("path-browserify");
const vue_extract_file_1 = require("../plugins/vue-extract-file");
const types_1 = require("../types");
function create(ts) {
    return {
        create(context) {
            let casing = types_1.TagNameCasing.Pascal; // TODO
            return {
                async provideDocumentDropEdits(document, _position, dataTransfer) {
                    if (document.languageId !== 'html')
                        return;
                    const [virtualFile, sourceFile] = context.language.files.getVirtualFile(document.uri);
                    const vueFile = sourceFile?.virtualFile?.[0];
                    if (!virtualFile || !(vueFile instanceof language_core_1.VueFile))
                        return;
                    let importUri;
                    for (const [mimeType, item] of dataTransfer) {
                        if (mimeType === 'text/uri-list') {
                            importUri = item.value;
                        }
                    }
                    if (!importUri?.endsWith('.vue'))
                        return;
                    let baseName = importUri.substring(importUri.lastIndexOf('/') + 1);
                    baseName = baseName.substring(0, baseName.lastIndexOf('.'));
                    const newName = (0, shared_1.capitalize)((0, shared_1.camelize)(baseName));
                    let additionalEdit;
                    for (const file of (0, language_core_1.forEachEmbeddedFile)(vueFile)) {
                        if ((file.languageId === 'typescript'
                            || file.languageId === 'javascript'
                            || file.languageId === 'typescriptreact'
                            || file.languageId === 'javascriptreact')
                            && file.mappings.some(mapping => (0, language_core_1.isFoldingRangesEnabled)(mapping.data))) {
                            additionalEdit ??= {};
                            additionalEdit.changes ??= {};
                            additionalEdit.changes[file.id] = [];
                            const { sfc } = vueFile;
                            const script = sfc.scriptSetup ?? sfc.script;
                            if (!sfc.template || !script)
                                return;
                            const lastImportNode = (0, vue_extract_file_1.getLastImportNode)(ts, script.ast);
                            additionalEdit.changes[file.id].push({
                                range: lastImportNode ? {
                                    start: document.positionAt(lastImportNode.end),
                                    end: document.positionAt(lastImportNode.end),
                                } : {
                                    start: document.positionAt(0),
                                    end: document.positionAt(0),
                                },
                                newText: `\nimport ${newName} from './${path.relative(path.dirname(document.uri), importUri) || importUri.substring(importUri.lastIndexOf('/') + 1)}'`
                                    + (lastImportNode ? '' : '\n'),
                            });
                            if (sfc.script) {
                                const edit = (0, vue_extract_file_1.createAddComponentToOptionEdit)(ts, sfc.script.ast, newName);
                                if (edit) {
                                    additionalEdit.changes[file.id].push({
                                        range: {
                                            start: document.positionAt(edit.range.start),
                                            end: document.positionAt(edit.range.end),
                                        },
                                        newText: edit.newText,
                                    });
                                }
                            }
                        }
                    }
                    return {
                        insertText: `<${casing === types_1.TagNameCasing.Kebab ? (0, shared_1.hyphenate)(newName) : newName}$0 />`,
                        insertTextFormat: 2,
                        additionalEdit,
                    };
                },
            };
        },
    };
}
exports.create = create;
//# sourceMappingURL=vue-document-drop.js.map